package mvd.jester.tests;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import mvd.jester.model.SortedTaskSet;
import mvd.jester.model.SystemSetup;

/**
 * TestEnvironment
 */
public class TestEnvironment {

    private final long numberOfTaskSets;
    private final long numberOfProcessors;
    private final Set<Class<? extends AbstractTest>> abstractTests;

    public TestEnvironment(long numberOfTaskSets, long numberOfProcessors) {
        this.numberOfTaskSets = numberOfTaskSets;
        this.numberOfProcessors = numberOfProcessors;
        abstractTests = new HashSet<>();
    }

    public TestEnvironment registerTestInterface(Class<? extends AbstractTest> abstractTest) {
        abstractTests.add(abstractTest);

        return this;
    }

    public void runTests() {
        Map<AbstractTest, Long> testCases = new HashMap<>();

        for (Class<? extends AbstractTest> t : abstractTests) {
            try {
                testCases.put(t.newInstance(), (long) 0);
            } catch (Exception e) {
                System.out.println("Ahhh SHIT!");
            }
        }


        long checkedTasksets = 0;

        while (checkedTasksets < 4000) {
            SystemSetup.Generator generator = new SystemSetup.Generator();
            SystemSetup systemSetup = generator.build();

            double utilization = systemSetup.getUtilization();

            while (utilization <= systemSetup.getNumberOfProcessors()
                    && checkedTasksets < numberOfTaskSets) {
                checkedTasksets++;

                for (AbstractTest t : testCases.keySet()) {
                    if (t.runSchedulabilityCheck(systemSetup)) {
                        testCases.computeIfPresent(t, (k, v) -> v + 1);
                    }
                }
                generator.addTask(systemSetup);

                utilization = systemSetup.getUtilization();
            }
        }


        for (AbstractTest t : testCases.keySet()) {
            System.out.println("Testcase " + t.getName() + " scheduled " + testCases.get(t) + " of "
                    + numberOfTaskSets + " tasks!");
        }


    }
}
